// SPDX-License-Identifier: Apache-2.0
// Copyright Contributors to the OpenTimelineIO project

#pragma once

#define OPENTIME_VERSION v1_0

namespace opentime {
namespace OPENTIME_VERSION {
}

using namespace OPENTIME_VERSION;
} // namespace opentime
